<?php defined('SYSPATH') OR die('No direct access allowed.');
/**
 * Traps controller
 *
 *  op5, and the op5 logo are trademarks, servicemarks, registered servicemarks
 *  or registered trademarks of op5 AB.
 *  All other trademarks, servicemarks, registered trademarks, and registered
 *  servicemarks mentioned herein may be the property of their respective owner(s).
 *  The information contained herein is provided AS IS with NO WARRANTY OF ANY
 *  KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A
 *  PARTICULAR PURPOSE.
 */
class Traps_Controller extends Authenticated_Controller
{
	protected function is_authorized_to_edit_service($host, $service) {
		if (Auth::instance()->authorized_for('service_edit_all'))
			return true;
		if (!Auth::instance()->authorized_for('service_edit_contact'))
			return false;

		$pool = new ServicePool_Model();
		$set = $pool->all()->reduce_by('host.name', $host, '=')->reduce_by('description', $service, '=');
		# "but just check if the service can be retrieved - if so, we have permissions!"
		# service_edit_contact + service_view_all. Your move.
		$obj = $set->it(array('contacts'), array(), false, false)->current();
		if (!$obj)
			return false;
		if (in_array(Auth::instance()->get_user()->get_username(), $obj->get_contacts(), true))
			return true;
		return false;
	}

	private function json_fail($msg) {
		$this->template = new View('json');
		$this->template->success = false;
		$this->template->value = $msg;
	}

	private function json_ok($msg) {
		$this->template = new View('json');
		$this->template->success = true;
		$this->template->value = $msg;
	}

	private function get_param_or_json_fail($name) {
		$val = $this->input->post($name);
		if ($val === null) {
			$this->json_fail("Missing $name parameter");
			return false;
		}
		return $val;
	}

	/**
	 * Removes trap-related objects.
	 */
	public function delete() {
		if (!Auth::instance()->authorized_for('manage_trapper'))
			return $this->json_fail('You are not authorized to manage trapper');
		if (($type = $this->get_param_or_json_fail('type')) === false)
			return;
		if (($id = $this->get_param_or_json_fail('id')) === false)
			return;

		// Check that $type is valid
		$valid_types = array('matcher', 'module', 'handler', 'message', 'log');
		if (!in_array($type, $valid_types, true))
			return $this->json_fail("Incorrect type '$type' used in delete.");

		// 'message' really means 'log'
		if ($type == 'message')
			$type = 'log';

		// Fetch instance by type.
		$trapobj = call_user_func(
			array('Trap' . ucfirst($type) . 'Pool_Model', 'fetch_by_key'),
			$id
		);

		if (!$trapobj)
			return $this->json_fail("Nothing found for '$type' with id '$id'.");

		$trapobj->delete();
		$this->json_ok(ucfirst($type) . ' deleted');
	}

	/**
	 * Delete trap subservice.
	 */
	public function delete_service() {
		if (($host = $this->get_param_or_json_fail('host')) === false)
			return;
		if (($service = $this->get_param_or_json_fail('service')) === false)
			return;
		if (($tag = $this->get_param_or_json_fail('tag')) === false)
			return;

		if (!$this->is_authorized_to_edit_service($host, $service))
			return $this->json_fail('You are not authorized to edit this service');

		// Delete the trap subservice.
		TrapPool_Model::all()
			->reduce_by('host', $host, '=')
			->reduce_by('service', $service, '=')
			->reduce_by('tag', $tag, '=')
			->delete();

		$this->json_ok('Subservice deleted');
	}

	/**
	 * Change trap state.
	 */
	public function change_state() {
		if (($host = $this->get_param_or_json_fail('host')) === false)
			return;
		if (($service = $this->get_param_or_json_fail('service')) === false)
			return;
		if (($tag = $this->get_param_or_json_fail('tag')) === false)
			return;
		if (($state = $this->get_param_or_json_fail('state')) === false)
			return;

		if (!$this->is_authorized_to_edit_service($host, $service))
			return $this->json_fail('You are not authorized to edit this service');

		$trap = TrapPool_Model::all()
			->reduce_by('host', $host, '=')
			->reduce_by('service', $service, '=')
			->reduce_by('tag', $tag, '=')
			->one();

		if (!($trap instanceof Trap_Model)) {
			$this->json_fail(
				"Unknown trap for host: $host, service: $service, tag: $tag"
			);
		}

		$trap->change_state($state);
		$this->json_ok('State changed');
	}

	/**
	 * generate a fake trap from localhost and submit
	 */
	public function submit_trap() {
		if (!Auth::instance()->authorized_for('manage_trapper'))
			return $this->json_fail('You are not authorized to manage trapper');

		$version = $this->input->post('trap-version-selector');
		$args = array();
		if ($version == 'v1') {
			$args[] = escapeshellarg($this->input->post('v1-enterprise-oid'));
			$args[] = escapeshellarg($this->input->post('v1-agent'));
			$args[] = escapeshellarg($this->input->post('v1-generic-trap'));
			$args[] = escapeshellarg($this->input->post('v1-specific-trap'));
			$args[] = escapeshellarg($this->input->post('v1-uptime'));
		} elseif ($version == 'v2c') {
			$args[] = escapeshellarg($this->input->post('v2-uptime'));
			$args[] = escapeshellarg($this->input->post('v2-trap-oid'));
		}
		$oids = $this->input->post('oid');
		$types = $this->input->post('type');
		$value = $this->input->post('value');
		for ($i = 0; $i < count($oids); $i++) {
			$args[] = escapeshellarg($oids[$i]);
			$args[] = escapeshellarg($types[$i]);
			$args[] = escapeshellarg($value[$i]);
		}

		$cmd = 'snmptrap -Le -v '.substr($version, 1).' -c public localhost '.implode(' ', $args).' 2>&1';
		exec($cmd, $output, $return_var);

		if ($return_var) {
			$msg = $output ? implode("\n", $output) : 'Unknown error';
			return $this->json_fail("Command '$cmd' failed. Details: $msg");
		}
		$this->json_ok('Trap submitted');
	}

	/**
	 * Upload new trap handler, or update an existing one.
	 */
	public function upload_handler() {
		if (!Auth::instance()->authorized_for('manage_trapper'))
			return $this->json_fail('You are not authorized to manage trapper');

		if (($name = $this->get_param_or_json_fail('handler_name')) === false)
			return;

		$input = false;
		if (isset($_FILES['handler_file'])) {
			if ($_FILES['handler_file']['error'])
				return $this->json_fail('Upload failed');
			if ($_FILES['handler_file']['size'] > 65536) {
				return $this->json_fail(
					'The uploaded file was too big. If you still want to use ' .
					'it you can upload the handle through traped.'
				);
			}
			$input = $_FILES['handler_file']['tmp_name'];
		}

		$id = $this->input->post('handler_id');
		if (empty($id)) {
			if (strval($input) === '')
				return $this->json_fail("No file specified, and wasn't an update");

			// Create new handler.
			$set = TrapHandlerPool_Model::all()->reduce_by('name', $name, '=');
			if ($set->one())
				return $this->json_fail("A module already exists with that name");

			$handler = new TrapHandler_Model();

		} else {
			// Update existing handler.
			$handler = TrapHandlerPool_Model::fetch_by_key($id);
			if (!$handler)
				return $this->json_fail("No handler found with ID '$id'.");
		}

		if ($input) {
			$script = file_get_contents($input);
			if ($script === false)
				return $this->json_fail("Could not read file '$input'.");
			$handler->set_script($script);
		}
		$handler->set_name($name);
		$handler->save();
		$this->json_ok('Handler created/updated');
	}

	/**
	 * Upload new or update an existing module
	 */
	public function upload_module()	{
		if (!Auth::instance()->authorized_for('manage_trapper'))
			return $this->json_fail('You are not authorized to manage trapper');
		if (($name = $this->get_param_or_json_fail('module_name')) === false)
			return;

		$input = false;
		if (isset($_FILES['module_file'])) {
			if ($_FILES['module_file']['error'])
				return $this->json_fail('Upload failed');
			if ($_FILES['module_file']['size'] > 65536) {
				return $this->json_fail(
					'The uploaded file was too big. If you still want to use ' .
					'it you can upload the handle through traped.'
				);
			}
			$input = $_FILES['module_file']['tmp_name'];
		}

		$id = $this->input->post('module_id');
		if (empty($id)) {
			if (strval($input) === '')
				return $this->json_fail("No file specified, and wasn't an update");

			// Create new module.
			$set = TrapModulePool_Model::all()->reduce_by('name', $name, '=');
			if ($set->one())
				return $this->json_fail("A module already exists with that name");

			$module = new TrapModule_Model();
		} else {
			// Update existing module.
			$module = TrapModulePool_Model::fetch_by_key($id);
			if (!$module)
				return $this->json_fail("No module found with ID '$id'.");
		}

		if ($input) {
			$script = file_get_contents($input);
			if ($script === false)
				return $this->json_fail("Could not read file '$input'.");
			$module->set_script($script);
		}
		$module->set_name($name);
		$module->save();
		$this->json_ok('Module created/updated');
	}

	/**
	 * Upload new or update an existing matcher.
	 */
	public function upload_matcher() {
		if (!Auth::instance()->authorized_for('manage_trapper'))
			return $this->json_fail('You are not authorized to manage trapper');

		if (($oid = $this->get_param_or_json_fail('matcher_oid')) === false)
			return;
		if (($handler = $this->get_param_or_json_fail('matcher_handler')) === false)
			return;

		if (!preg_match('/^([.*0-9]+|fallback)$/', $oid)) {
			$this->json_fail(
				'Invalid match pattern supplied: must be an oid, optionally ' .
				'with the wildcard character "*", or the string "fallback". ' .
				"Oid was '$oid'."
			);
			return;
		}

		$n = $this->input->post('matcher_n');
		if (empty($n)) {
			// Create matcher.
			$matcher = new TrapMatcher_Model();
		} else  {
			$matcher = TrapMatcherPool_Model::fetch_by_key($n);
			if (!$matcher)
				return $this->json_fail("Could not find matcher with n '$n'.");
		}

		$matcher->set_oid($oid);
		$matcher->set_handler_id($handler);
		$matcher->save();
		$this->json_ok('Matcher created/updated');
	}

	/**
	 * Change position of a matcher.
	 */
	public function move_matcher() {
		if (!Auth::instance()->authorized_for('manage_trapper'))
			return $this->json_fail('You are not authorized to manage trapper');
		if (($n = $this->get_param_or_json_fail('n')) === false)
			return;
		if (($direction = $this->get_param_or_json_fail('direction')) === false)
			return;
		if ($direction !== 'up' && $direction !== 'down')
			return $this->json_fail('Direction should be "up" or "down".');

		$all_matchers = TrapMatcherPool_Model::all();
		$matchers = $all_matchers->reduce_by('n', $n, $direction == 'down' ? '>' : '<');

		if (count($matchers) == 0)
			return $this->json_ok('Nowhere to move matcher');

		// Sort matchers on n since we cannot get $matchers sorted from ORM.
		$matchers = iterator_to_array($matchers);
		usort($matchers, function($a, $b) use ($direction) {
			if ($direction == 'down')
				return $a->get_n() < $b->get_n() ? -1 : 1;
			return $a->get_n() > $b->get_n() ? -1 : 1;
		});

		// Let $b be the matcher that we want to swap position with.
		$b = $matchers[0];
		$new_n = $b->get_n();

		// Set $b to temp position. We must do this since n is a unique index
		// in the DB table.
		$b->set_n(TrapMatcher_Model::gen_tmp_n());
		$b->save();

		// Let $a be the matcher we are moving.
		$a = $all_matchers->reduce_by('n', $n, '=')->one();
		$old_n = $a->get_n();

		// Move $a to its new position.
		$a->set_n($new_n);
		$a->save();

		// Set $b to its new position.
		$b->set_n($old_n);
		$b->save();

		$this->json_ok('Matcher moved ' . $direction);
	}

	/**
	 * Download lua code as a text file. The listview only gives you the top snippet.
	 */
	public function download_code($type, $id) {
		if (!Auth::instance()->authorized_for('manage_trapper')) {
			$this->template = new View('401');
			$this->template->messages = array();
			$this->template->messages[] =
				'You are not authorized to manage trapper';
			return;
		}

		switch ($type) {
			case 'handler':
				$obj = TrapHandlerPool_Model::fetch_by_key($id);
				break;
			case 'module':
				$obj = TrapModulePool_Model::fetch_by_key($id);
				break;
			default:
				$this->template = new View('500');
				$this->template->error_message = "Invalid type '$type'.";
				return;
		}

		if (!$obj) {
			$this->template = new View('500');
			$this->template->error_message =
				"Could not find a '$type' with ID '$id'.";
			return;
		}

		$this->template = new View('text');
		$this->template->filename = $obj->get_name() . '.lua';
		$this->template->content = $obj->get_script();
	}
}
