<?php

/**
 * Override database and authorization from parent model
 */
class TrapSet_Model extends BaseTrapSet_Model {
	/**
	 * Return resource name of this object
	 * @return string
	 */
	public function mayi_resource() {
		return "monitor.trapper.traps";
	}

	/**
	 * Get statistics of the hosts in the set
	 */
	public function get_totals() {
		$logp = new TrapLogPool_Model();
		$pool = new TrapLogPool_Model();
		$stats = array(
			'subservices_all' => $pool->get_by_query('[traps] all'),
			'subservices_state_ok' => $pool->get_by_query('[traps] state = 0'),
			'subservices_state_warning' => $pool->get_by_query('[traps] state = 1'),
			'subservices_state_critical' => $pool->get_by_query('[traps] state = 2'),
			'subservices_state_unknown' => $pool->get_by_query('[traps] state = 3'),
			'trap_errors' => $logp->all(),
		);

		$totals = array();
		foreach ($stats as $name => $set) {
			$totals[$name] = array($set->get_query(), $set->count());
		}

		return $totals;
	}

	protected function get_auth_filter() {
		$auth = Auth::instance();
		$all_services = $auth->authorized_for('service_view_all');

		// Authorized for everything? Don't filter anything.
		// This means service_view_all is also required to see
		// unmatched services.
		if ($all_services)
				return $this->filter;

		//Authorized for traps_view_all
		if ($auth->authorized_for('traps_view_all'))
			return $this->filter;

		// Otherwise Darn.
		$contact_name = $auth->get_user()->get_username();

		// If we think there are more services the user is authorized for than
		// trapper services (reasonable, no?), it should be quicker to first
		// retrieve the trapper services, then reduce the servicepool by that.
		// This should make the SQL query smaller, but the previous livestatus
		// query bigger.
		// A question of indexes and mysql-vs-livestatus also affect the optimal
		// performance - more benchmarking is needed.
		$set = ServicePool_Model::all();

		$auth_filter = new LivestatusFilterOr();
		foreach ($set->it(array('host.name', 'description'), array(), false, false) as $svc) {
			$obj = new LivestatusFilterAnd();
			$obj->add(new LivestatusFilterMatch('host', $svc->get_host()->get_name()));
			$obj->add(new LivestatusFilterMatch('service', $svc->get_description()));
			$auth_filter->add($obj);
		}

		$result_filter = new LivestatusFilterAnd();
		$result_filter->add($this->filter);
		$result_filter->add($auth_filter);
		return $result_filter;
	}
}
