listview_renderer_table.trap_modules = {
	'name': {
		'header': 'Name',
		'depends': ['name'],
		'sort': ['name'],
		'cell': function(args) {
			return $('<td />').update_text(args.obj.name);
		}
	},
	"actions" : {
		"header" : _('Actions'),
		"depends" : ['id', 'name'],
		"sort" : false,
		"cell" : function(args) {
			var cell = $('<td />');
			cell.append(icon16('remove',
					_('Delete module'), $('<a href="#"/>')).data('id', args.obj.id).data('type', 'module').addClass('trap_delete').attr('title',  _('Delete module')));
			cell.append(icon16('add-comment',
					_('Edit module'), $('<a href="#"/>')
			).data('module_id', args.obj.id).data('module_name', args.obj.name).data('drop-down', 'filter-query-upload-module').addClass('trap_edit').attr('title',  _('Edit module')));
			return cell;
		}
	},
	'script': {
		'header': 'Script',
		'depends': ['script', 'id'],
		'sort': ['script'],
		'cell': function(args) {
			var maxlines = 2;
			var lines = $.trim(args.obj.script).split('\n');
			var longer = false;
			if (lines.length > maxlines)
				longer = true;
			var res = $('<code/>');
			for (var i = 0; i < maxlines; i++) {
				res.append($('<span/>').update_text(lines[i]));
				if (longer || i + 1 < maxlines)
					res.append('<br />');
			}
			if (longer)
				res.append($('<a />').attr('href', _site_domain + _index_page + '/traps/download_code/module/' + args.obj.id).append('Download'));
			return $('<td/>').append(res);
		}
	}
};

listview_renderer_table.trap_handlers = {
	'name': {
		'header': 'Name',
		'depends': ['name'],
		'sort': ['name'],
		'cell': function(args) {
			return $('<td />').update_text(args.obj.name);
		}
	},
	'matchers': {
		'header': 'Matchers',
		'depends': ['oid', 'id'],
		'sort': ['oid'],
		'cell': function(args) {
			return $('<td />').html(link_query('[trap_matchers] handler_id = ' + args.obj.id).html(args.obj.oid.replace(/\n/g, "<br/>")));
		}
	},
	"actions" : {
		"header" : _('Actions'),
		"depends" : ['id', 'name'],
		"sort" : false,
		"cell" : function(args) {
			var cell = $('<td />');
			cell.append(icon16('remove',
					_('Delete handler'), $('<a href="#"/>')).data('id', args.obj.id).data('type', 'handler').addClass('trap_delete').attr('title',  _('Delete handler')));
			cell.append(icon16('add-comment',
					_('Edit handler'), $('<a href="#"/>')
			).data('handler_id', args.obj.id).data('handler_name', args.obj.name).data('drop-down', 'filter-query-upload-handler').addClass('trap_edit').attr('title',  _('Edit handler')));
			return cell;
		}
	},
	'script': {
		'header': 'Script',
		'depends': ['script', 'id'],
		'sort': ['script'],
		'cell': function(args) {
			var maxlines = 2;
			var lines = $.trim(args.obj.script).split('\n');
			var longer = false;
			if (lines.length > maxlines)
				longer = true;
			var res = $('<code/>');
			for (var i = 0; i < maxlines; i++) {
				res.append($('<span/>').update_text(lines[i]));
				if (longer || i + 1 < maxlines)
					res.append('<br />');
			}
			if (longer)
				res.append($('<a />').attr('href', _site_domain + _index_page + '/traps/download_code/handler/' + args.obj.id).append('Download'));
			return $('<td/>').append(res);
		}
	},
	'context': {
		'header': 'Context',
		'depends': ['context'],
		'sort': ['context'],
		'cell': function(args) {
			return $('<td/>').html('<code>'+args.obj.context+'</code>');
		},
	}
};

listview_renderer_table.trap_matchers = {
	'n': {
		'header': 'Matching order',
		'depends': ['n'],
		'sort': ['n'],
		'cell': function(args) {
			return $('<td />').update_text(args.obj.n);
		}
	},
	'handlers': {
		'header': 'Handler',
		'depends': ['handlers.name', 'handler_id'],
		'sort': ['handlers.name'],
		'cell': function(args) {
			return $('<td />').html(link_query('[trap_handlers] id = ' + args.obj.handler_id).update_text(args.obj.handlers.name));
		}
	},
	'oid': {
		'header': 'OID',
		'depends': ['oid'],
		'sort': ['oid'],
		'cell': function(args) {
			return $('<td />').update_text(args.obj.oid);
		}
	},
	"actions" : {
		"header" : _('Actions'),
		"depends" : ['n', 'handler_id', 'oid'],
		"sort" : false,
		"cell" : function(args) {
			var cell = $('<td />');
			cell.append(icon16('remove',
					_('Delete matcher'), $('<a href="#" data-type="matcher" class="trap_delete"/>')
				).data('id', args.obj.n).attr('title', _('Delete matcher')));
			cell.append(icon16('add-comment',
					_('Edit matcher'), $('<a href="#" data-type="matcher" class="trap_edit"/>')
				).data('matcher_n', args.obj.n).data('matcher_handler', args.obj.handler_id).data('matcher_oid', args.obj.oid).data('drop-down', 'filter-query-set-matcher').attr('title', _('Edit matcher')));
			cell.append(icon16('arrow-up',
					_('Move matcher up'), $('<a href="#" data-type="matcher" data-direction="up" class="trap_matcher_move"/>')
				).data('n', args.obj.n).attr('title', _('Move matcher up')));
			cell.append(icon16('arrow-down',
					_('Move matcher down'), $('<a href="#" data-type="matcher" data-direction="down" class="trap_matcher_move"/>')
				).data('n', args.obj.n).attr('title', _('Move matcher down')));
			return cell;
		}
	}
};

listview_renderer_table.trap_logs = {
	'time': {
		'header': 'Time',
		'depends': ['time'],
		'sort': ['time'],
		'cell': function(args) {
			return $('<td />').update_text(args.obj.time);
		}
	},
	'handler': {
		'header': 'Handler',
		'depends': ['handler'],
		'sort': ['handler'],
		'cell': function(args) {
			return $('<td />').update_text(args.obj.handler);
		}
	},
	'message': {
		'header': 'Message',
		'depends': ['message'],
		'sort': ['message'],
		'cell': function(args) {
			return $('<td />').update_text(args.obj.message);
		}
	},
	"actions" : {
		"header" : _('Actions'),
		"depends" : ['id'],
		"sort" : false,
		"cell" : function(args) {
			var cell = $('<td />');
			cell.append(icon16('remove',
					_('Delete log entry'), $('<a href="#" data-type="message" class="trap_delete"/>')
				).data('id', args.obj.id).attr('title', _('Delete log entry')));
			return cell;
		}
	}
};

listview_renderer_table.traps = {
	"state" : {
		"header" : '',
		"depends" : [ 'state', 'name' ],
		"sort" : [ 'state desc' ],
		"cell" : function(args) {
			state_text = ['ok', 'warning', 'critical', 'unknown'][args.obj.state];
			return $('<td class="icon obj_properties" />')
					.append(
							icon16('shield-' + state_text,
									state_text)).addClass(
							state_text);
		}
	},
	"host" : {
		"header" : _('Host'),
		"depends" : [ 'host', 'matches_service' ],
		"sort" : [ 'host', 'service' ],
		"cell" : function(args) {
			var cell = $('<td />');
			var text = args.obj.host;
			if (args.obj.matches_service) {
				text = extinfo_link({
					host : args.obj.host
					}).update_text(text);
			}
			cell.append(text);
			return cell;
		}
	},
	"service" : {
		"header" : _('Service'),
		"depends" : [ 'host', 'service', 'matches_service' ],
		"sort" : [ 'service' ],
		"cell" : function(args) {
			var cell = $('<td />');
			var text = args.obj.service;
			if (args.obj.matches_service) {
				text = extinfo_link({
					host : args.obj.host,
					service : args.obj.service
				}).update_text(text);
			}
			else {
				text += "<br/><small>(doesn't exist)</small>";
			}
			cell.append(text);
			return cell;
		}
	},
	"tag" : {
		"header" : _('Subservice Tag'),
		"depends" : [ 'tag' ],
		"sort" : [ 'tag' ],
		"cell" : function(args) {
			return $('<td />').update_text(args.obj.tag);
		}
	},
	"actions" : {
		"header" : _('Actions'),
		"depends" : ['host', 'service', 'tag'],
		"sort" : false,
		"cell" : function(args) {
			var cell = $('<td />');
			cell.append(
				icon16('remove', _('Delete subservice'), $('<a />').attr('href', '#'))
					.data('host', args.obj.host)
					.data('service', args.obj.service)
					.data('tag', args.obj.tag)
					.addClass('trap_delete_service')
					.attr('title',  _('Delete subservice')));
			cell.append(
				icon16('settings', _('Change subservice state'), $('<a />').attr('href', '#'))
					.data('host', args.obj.host)
					.data('service', args.obj.service)
					.data('tag', args.obj.tag)
					.addClass('change_state')
					.attr('title', _('Change subservice state')));

			return cell;
		}
	},
	"time" : {
		"header" : _('Time'),
		"depends" : [ 'time' ],
		"sort" : [ 'time' ],
		"cell" : function(args) {
			return $('<td />').text(args.obj.time);
		}
	},
	"count" : {
		"header" : _('Count'),
		"depends" : [ 'count' ],
		"sort" : [ 'count' ],
		"cell" : function(args) {
			return $('<td />').update_text(args.obj.count);
		}
	},
	"oid" : {
		"header" : _('OID'),
		"depends" : [ 'trapoid' ],
		"sort" : [ 'trapoid' ],
		"cell" : function(args) {
			return $('<td />').update_text(args.obj.trapoid);
		}
	},
	"message" : {
		"header" : _('Message'),
		"depends" : [ 'message' ],
		"sort" : [ 'message' ],
		"cell" : function(args) {
			return $('<td />').update_text(args.obj.message);
		}
	},
	"trap" : {
		"header" : _('Raw Data'),
		"depends" : [ 'trap' ],
		"sort" : [ 'trap' ],
		"cell" : function(args) {
			return $('<td />').update_text(args.obj.trap);
		}
	}
};
