function Trapper() {
	var that = this;
	var on_success = function(data) {
		if (data) {
			$.notify(data, {'type': 'success'});
			lsfilter_main.refresh();
		}
		else {
			$.notify('An unexpected error occured', {'sticky':true, 'type':'error'});
		}
	};
	var on_error = function(cb) {
		return function(data) {
			var response;
			try {
				response = $.parseJSON(arguments[0].responseText);
			} catch (err) {
				response = "Unknown error";
			}
			return cb(response);
		};
	};
	var load_handlers = function(cb) {
		return function() {
			var that = this;
			if ($('#matcher_handler option').length > 1) {
				if (cb) {
					cb.call(that);
				}
				return;
			}
			$.ajax({
				url: _site_domain + _index_page + '/listview/fetch_ajax',
				data: {
					query: '[trap_handlers] all',
					'columns': ['name', 'id'],
					'limit': 10000,
					csrf_token: _csrf_token
				},
				success: function(data) {
					$.each(data.data, function(idx, innerdata) {
						$('#matcher_handler').append(
							$('<option />')
								.val(innerdata.id)
								.text(innerdata.name)
						);});
					if (cb)
						cb.call(that);
				},
				dataType: 'json'
			});
		};
	};

	var load_form_data = function() {
		var that = this;
		var dd = $('#' + $(this).data('drop-down'));
		var form = dd.find('form');
		if (!form) {
			return;
		}
		form = form.get(0);
		for (var i = 0; i < form.length; i++) {
			var value = $(this).data(form[i].name);
			if (value) {
				$(form[i]).val(value);
			}
		}
		dd.show();
		$('[data-drop-down='+dd.attr('id')+']').addClass('active');
	};

	create_change_state_popup = function (element) {
		if ($('#change_state').length)
			that.destroy_change_state_popup();

		$('body').append('<div id="change_state" style="position: absolute; left: ' +
			element.offset().left + 'px; top: ' + (element.offset().top + element.outerHeight()) +
			'px; padding: 5px 10px; line-height: 1.5em; border: 1px solid #999999; background-color: #F5F5F5;">' +
			'<a href="#"><span class="status-ok">Ok</span></a><br />' +
			'<a href="#"><span class="status-warning">Warning</span></a><br />' +
			'<a href="#"><span class="status-critical">Critical</span></a><br />' +
			'<a href="#"><span class="status-unknown">Unknown</span></a></div>');
		$('body, #change_state a').one('click', function() {
			$('#change_state').remove();
		});
	};
	$('.content').on('click', '.change_state', function(){
		var that = this;
		create_change_state_popup($(this));
		$('#change_state a').click(function(){
			var states = {Ok: 0, Warning: 1, Critical: 2, Unknown: 3};
			var data = {};
			data.host = $(that).data('host');
			data.service = $(that).data('service');
			data.tag = $(that).data('tag');
			data.state = states[$(this).text()]
			data.csrf_token = _csrf_token;

			$.ajax({
				url: _site_domain + _index_page + '/traps/change_state/',
				data: data,
				type: 'POST',
				success: on_success,
				error: on_error(function(msg){$.notify("Failed to change state: "+msg, {'sticky':true, 'type':'error'});}),
				dataType: 'json'
			});
			return false;
		});
		return false;
	});

	$('.content').on('click', '.trap_delete', function() {
		var that = this;
		if (confirm("Really delete "+$(that).data('type')+"?"))
		{
			var type = $(that).data('type');
			var data = $(that).data();
			data.csrf_token = _csrf_token;
			$.ajax({
				url: _site_domain + _index_page + '/traps/delete/',
				data: data,
				type: 'POST',
				success: on_success,
				error: on_error(function(msg){$.notify("Failed to delete: "+msg, {'sticky':true, 'type':'error'});}),
				dataType: 'json'
			});
		}
		return false;
	});
	
	$('.content').on('click', '.trap_delete_service', function() {
		var that = this;
		if (confirm("Do you really want to delete this trap?"))	{
			var data = $(that).data();
			data.csrf_token = _csrf_token;
			$.ajax({
				url: _site_domain + _index_page + '/traps/delete_service/',
				data: {
					host:    $(that).data('host'),
					service: $(that).data('service'),
					tag:     $(that).data('tag'),
					csrf_token: _csrf_token
				},
				type: 'POST',
				success: on_success,
				error: on_error(function(msg){$.notify("Failed to delete: "+msg, {'sticky':true, 'type':'error'});}),
				dataType: 'json'
			});
		}
		return false;
	});

	$('.content').on('click', '.trap_edit:not([data-type="matcher"])', load_form_data);

	$('.main-toolbar-buttons').on('click', '#show-filter-query-trap-new-matcher', load_handlers());
	$('.content').on('click', '.trap_edit[data-type="matcher"]', load_handlers(load_form_data));

	$('.content').on('click', '.trap_matcher_move', function() {
		var that = this;
		var data = $(that).data();
		data.csrf_token = _csrf_token;
		$.ajax({
			url: _site_domain + _index_page + '/traps/move_matcher/',
			data: data,
			type: 'POST',
			success: on_success,
			error: on_error(function(msg){$.notify("Failed to move match: "+msg, {'sticky':true, 'type':'error'});}),
			dataType: 'json'
		});
	});

	$('.main-toolbar-buttons').on('click', '.extra_link', function () {
		lsfilter_main.update($(this).data('for'), false, '' );
	});

	$('#extra-dropdowns').on('change', '#trap-version-selector', function() {
		$('#v1, #v2c').hide();
		$('#'+this.value).show();
	});
	$('#extra-dropdowns').on('submit', '#trap-send-trap', function(e) {
		e.preventDefault();
		var data = {};
		data.csrf_token = _csrf_token;
		$(this).ajaxSubmit({
			type: 'POST',
			data: data,
			url: _site_domain + _index_page + '/traps/submit_trap/',
			success: on_success,
			error: on_error(function(msg){$.notify("Failed to send trap: "+msg, {'sticky':true, 'type':'error'});}),
			dataType: 'json'
		});
	});

	$('#extra-dropdowns').on('submit', '#trap-upload-handler', function(e) {
		e.preventDefault();
		var data = {};
		data.csrf_token = _csrf_token;
		$(this).ajaxSubmit({
			type: 'POST',
			data: data,
			url: _site_domain + _index_page + '/traps/upload_handler/',
			enctype: 'multipart/form-data',
			success: on_success,
			error: on_error(function(msg){$.notify('Failed to upload trap handler: '+msg, {'sticky':true, 'type':'error'});}),
			dataType: 'json',
		});
	});

	$('#extra-dropdowns').on('submit', '#trap-upload-module', function(e) {
		e.preventDefault();
		var data = {};
		data.csrf_token = _csrf_token;
		$(this).ajaxSubmit({
			type: 'POST',
			data: data,
			url: _site_domain + _index_page + '/traps/upload_module/',
			enctype: 'multipart/form-data',
			success: on_success,
			error: on_error(function(msg){$.notify('Failed to upload trap module: '+msg, {'sticky':true, 'type':'error'});}),
			dataType: 'json',
		});
	});

	$('#extra-dropdowns').on('submit', '#trap-upload-matcher', function(e) {
		e.preventDefault();
		var data = {};
		data.csrf_token = _csrf_token;
		$(this).ajaxSubmit({
			type: 'POST',
			data: data,
			url: _site_domain + _index_page + '/traps/upload_matcher/',
			success: on_success,
			error: on_error(function(msg){$.notify('Failed to set trap matcher: '+msg, {'sticky':true, 'type':'error'});}),
			dataType: 'json',
		});
	});

	$('#extra-dropdowns').on('click', '.trap-delete', function() {$(this).parent().remove();});
	$('#extra-dropdowns').on('click', '.trap-add', function() {
		Trapper.add_oid($('#extra-dropdowns'));
	});

	destroy_change_state_popup = function (element) {
		$('#change_state').remove();
		$('#change_state a').unbind('click');
	};

	$('#extra-dropdowns').on('change keyup', '#trap-send-trap input, #trap-send-trap select', function() {
		var text = 'snmptrap -Le -c public localhost';
		var version = $('[name=trap-version-selector]').val();
		if (version == 'v2c') {
			text += " -v2c " + $('[name=v2-uptime]').val() + ' ' + $('[name=v2-trap-oid]').val();
		} else {
			text += " -v1 " + $('[name=v1-enterprise-oid]').val() + ' ' + $('[name=v1-agent]').val() + ' ' + $('[name=v1-generic-trap]').val() + ' ' + $('[name=v1-specific-trap]').val() + ' ' + $('[name=v1-uptime]').val();
		}
		var ary = $('#trap-send-trap').serializeArray();
		for (var i in ary) {
			if (ary[i].name == 'oid[]' || ary[i].name == 'type[]' || ary[i].name == 'value[]') {
				text += ' ' + ary[i].value;
			}
		}

		$('.command-line').text(text);
	});
}

Trapper.add_oid = function (root) {
	var line = Trapper.line_template.clone();
	line.children().each(function() {
		var elem = $(this);
		$.each(['id', 'for'], function(id, attr) {
			if (elem.attr(attr)) {
				elem.attr(attr, elem.attr(attr) + '_' + Trapper.i);
			}
		});
		if (elem.attr('name')) {
			elem.attr('name', elem.attr('name') + '[]');
		}
	});
	Trapper.i++;
	root.find('#trap-content').append(line);
};

Trapper.custom_template_setup = function (root) {
	Trapper.line_template = root.find('.line-template');
	Trapper.i = 1;
	Trapper.line_template.remove();
	Trapper.line_template.attr('class', 'trap-line');
	Trapper.add_oid(root);
};

$(document).ready(function(){
	new Trapper();
});
